#!/bin/sh
# Copyright (C) 2014 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

# If everything else fails.. ;)
killall -q -9 VirtualBox VBoxSVC VBoxXPCOMIPCD VBoxHeadless VBoxNetDHCP
